const {
    request,
    unescapeEntity,
    success,
    error,
    writeFileSync
} = require("./core/index.js");
const Cheerio = require("./core/cheerio.js");
const Cache = require("./core/cache.js");
const config = require("./config.js");

const cheerio = new Cheerio();
const cache = new Cache();

const appConfig = {
    version: 1,
    categories: "https://cn.pornhub.com/categories",
    direction: "horizontal",
    typeMap: []
};

function calculatePages(str) {
    const size = parseInt(str.match(/(\d+)-(\d+)/)[2]);
    const total = parseInt(str.match(/共有(\d+)个/)[1]);
    const pages = Math.ceil(total / size);

    return { size, total, pages };
}

/**
 * 获取分类列表
 */
async function getTypeList({ type_id, page = 1 }) {
    if (cache.get(`type:${type_id}:page:${page}`)) {
        return cache.get(`type:${type_id}:page:${page}`);
    }

    const response = await request(config.list(type_id, page));
    const html = await response.text();

    const $ = cheerio.load(html);

    const pagesInfo = $(".showingCounter").text().trim();

    const { size, total, pages } = calculatePages(pagesInfo);

    const list = [];
    $("#videoCategory .pcVideoListItem").each((_, element) => {
        const $element = $(element).find(".phimage").find(".linkVideoThumb");

        const vod_id = $element.attr("href")?.match(/viewkey=([^&]+)/)?.[1];
        const vod_name = unescapeEntity($element.find("img").attr("alt"));
        const vod_pic = $element.find("img").attr("src");
        const vod_remarks = $(element).find(".duration").text().trim();
        const vod_direction = appConfig.direction;

        if (vod_id && vod_name && vod_pic) {
            list.push({
                vod_id,
                vod_name,
                vod_pic,
                vod_remarks,
                vod_direction
            });
        }
    });

    return {
        page,
        size,
        total,
        pages,
        type_id,
        list
    };
}

async function home() {
    try {
        // 获取分类信息
        const categoryResponse = await request(appConfig.categories);
        const categoryHtml = await categoryResponse.text();

        let $ = cheerio.load(categoryHtml);

        let categories = [];
        $("#categoriesListingWrapper > ul > li").each((_, element) => {
            const $element = $(element).find(".relativeWrapper").find("a");
            const type_id = $element.attr("href");
            const type_name = $element.attr("alt");

            if (type_id && type_name) {
                categories.push({
                    type_id,
                    type_name,
                    type_type: "other"
                });
            }
        });

        if (categories.length > 10) {
            categories.sort(() => 0.5 - Math.random());
            categories.length = 10;
        }

        const results = await Promise.allSettled(
            categories.map(async item => {
                return getTypeList({
                    type_id: item.type_id
                });
            })
        );

        results.forEach(async (item, index) => {
            if (item.status === "fulfilled") {
                if (item.value) {
                    categories[index]["vod_list"] = item.value.list;

                    cache.set(
                        `type:${categories[index]["type_id"]}:page:1`,
                        item.value
                    );
                }
            }
        });

        return success(categories.filter(item => item.vod_list));
    } catch (e) {
        // console.error('发生错误:', e)
        return error(e.message);
    }
}

async function detail({ vod_id }) {
    try {
        const detailResponse = await request(config.detail(vod_id));
        const notStandardHtml = await detailResponse.text();

        const $ = cheerio.load(notStandardHtml);

        const vod_name = $(".title-container")
            .find(".inlineFree")
            .text()
            .trim();
        const vod_pic = $("meta[property='og:image']").attr("content");
        const vod_remarks = $("meta[property='video:duration']")
            .attr("content")
            .trim();

        const vod_content =
            $(".description-container").text().trim() || vod_name;

        const categories = [];
        $(".categoriesWrapper a").each((_, element) => {
            categories.push($(element).text().trim());
        });
        const vod_class = categories.join(",") || "";

        const vod_year = $(".video-actions-menu").find(".videoInfo").text();

        // 提取相似视频列表
        const similar = [];
        $(".phimage").each((_, element) => {
            const $element = $(element).find(".linkVideoThumb");

            const video_href = $element.attr("href");
            const $thumb = $element.find("img");
            const vod_name = $thumb.attr("alt");
            const vod_pic = $thumb.attr("src");
            const vod_remarks = $(element).find(".duration").text().trim();
            const vod_id = video_href?.match(/viewkey=([^&]+)/)?.[1];

            if (vod_id && vod_name && vod_pic) {
                similar.push({ vod_id, vod_name, vod_pic, vod_remarks });
            }
        });

        // 提取播放链接
        const flashvarsMatch = notStandardHtml.match(
            /var\s+flashvars_\d+\s*=\s*({[\s\S]*?});/
        );
        let vod_play_list = [];

        if (flashvarsMatch) {
            try {
                const flashvars = JSON.parse(flashvarsMatch[1]);
                const mediaDefinitions = flashvars.mediaDefinitions || [];

                // 提取HLS格式的视频
                const hlsVideos = mediaDefinitions.filter(
                    media =>
                        media.format === "hls" &&
                        media.videoUrl &&
                        !media.remote
                );

                if (hlsVideos.length > 0) {
                    vod_play_list = hlsVideos.map(video => ({
                        name:
                            video.quality === "auto"
                                ? "自动"
                                : `${video.height}P`,
                        url: video.videoUrl
                    }));
                } else {
                    const otherVideos = mediaDefinitions.filter(
                        media => media.videoUrl && !media.remote
                    );

                    if (otherVideos.length > 0) {
                        vod_play_list = otherVideos.map(video => ({
                            name: video.height
                                ? `${video.height}P`
                                : video.quality || "未知",
                            url: video.videoUrl
                        }));
                    }
                }
            } catch (e) {
                console.error("解析播放链接失败:", e);
            }
        }

        // 构建播放源
        const vod_sources = [
            {
                source_name: "默认线路",
                vod_play_list: {
                    url_count: vod_play_list.length,
                    urls: vod_play_list
                }
            }
        ];

        const result = {
            vod_id,
            vod_name,
            vod_pic,
            vod_remarks,
            vod_content,
            vod_class,
            vod_actor: "",
            vod_area: "",
            vod_year,
            vod_time: "",
            vod_play_url: "",
            vod_sources,
            similar
        };

        return success(result);
    } catch (e) {
        console.error("解析视频详情失败:", e);
        return error(e.message);
    }
}

function similar(document) {
    return [];
}

/**
 * 获取视频播放地址
 * @param {Object} params
 * @param {string} params.url - 播放链接
 * @returns {Promise<Object>}
 */
async function play({ url }) {
    try {
        if (!url || !url.startsWith("http")) {
            throw new Error("无效的播放链接");
        }

        return success({
            url: url
        });
    } catch (e) {
        console.error("获取播放地址失败:", e);
        return error(e.message);
    }
}

async function list(type_id, page = 1) {
    const data = await getTypeList({
        type_id,
        page
    });

    return success(data);
}

async function search(keyword, page = 1) {
    try {
        const response = await request(config.search(keyword, page));
        const notStandardHtml = await response.text();

        const $ = cheerio.load(notStandardHtml);

        const list = [];
        $(".phimage").each((_, element) => {
            const $element = $(element).find(".linkVideoThumb");

            const video_href = $element.attr("href");
            const $thumb = $element.find("img");
            const vod_name = $thumb.attr("alt");
            const vod_pic = $thumb.attr("src");
            const vod_remarks = $(element).find(".duration").text().trim();
            const vod_id = video_href?.match(/viewkey=([^&]+)/)?.[1];

            if (vod_id && vod_name && vod_pic) {
                list.push({ vod_id, vod_name, vod_pic, vod_remarks });
            }
        });

        // 获取总页数
        let pages = 1;
        const $pagination = $(".pagination3");
        if ($pagination) {
            const lastPageLink = $pagination
                .find(".page_number")
                .last()
                .text()
                .trim();
            if (lastPageLink) {
                const pageNum = parseInt(lastPageLink);
                if (!isNaN(pageNum)) {
                    pages = pageNum;
                }
            }
        }

        return success({
            page: parseInt(page),
            pagecount: pages,
            limit: 50,
            total: pages * 50,
            list: list
        });
    } catch (e) {
        console.error("搜索失败:", e);
        return error(e.message);
    }
}

module.exports.home = home;
module.exports.list = list;
module.exports.search = search;
module.exports.detail = detail;
module.exports.similar = similar;
module.exports.play = play;
module.exports.getTypeList = getTypeList;
